@@ -1,9 +1,9 @@
 /*
  *  Phusion Passenger - https://www.phusionpassenger.com/
- *  Copyright (c) 2011-2018 Phusion Holding B.V.
+ *  Copyright (c) 2011-2025 Asynchronous B.V.
  *
  *  "Passenger", "Phusion Passenger" and "Union Station" are registered
- *  trademarks of Phusion Holding B.V.
+ *  trademarks of Asynchronous B.V.
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
@@ -44,25 +44,17 @@
 #include <boost/cstdint.hpp>
 #include <oxt/macros.hpp>
 #include <ev++.h>
-#include <ostream>
-
-#if defined(__GLIBCXX__) || defined(__APPLE__)
-	#include <cxxabi.h>
-	#define CXX_ABI_API_AVAILABLE
-#endif
 
 #include <sys/types.h>
 #include <sys/uio.h>
 #include <utility>
-#include <typeinfo>
+#include <exception>
 #include <cstdio>
 #include <cstdlib>
 #include <cstddef>
-#include <cassert>
-#include <cctype>
 
 #include <LoggingKit/LoggingKit.h>
-#include <MessageReadersWriters.h>
+#include <IOTools/MessageSerialization.h>
 #include <Constants.h>
 #include <ConfigKit/ConfigKit.h>
 #include <ServerKit/Errors.h>
@@ -74,9 +66,9 @@
 #include <WrapperRegistry/Registry.h>
 #include <StaticString.h>
 #include <Utils.h>
-#include <Utils/StrIntUtils.h>
-#include <Utils/IOUtils.h>
-#include <Utils/JsonUtils.h>
+#include <StrIntTools/StrIntUtils.h>
+#include <IOTools/IOUtils.h>
+#include <JsonTools/JsonUtils.h>
 #include <Utils/HttpConstants.h>
 #include <Utils/Timer.h>
 #include <Core/Controller/Config.h>
@@ -109,10 +101,6 @@ private:
 	typedef ServerKit::FileBufferedChannel FileBufferedChannel;
 	typedef ServerKit::FileBufferedFdSinkChannel FileBufferedFdSinkChannel;
 
-	// If you change this value, make sure that Request::sessionCheckoutTry
-	// has enough bits.
-	static const unsigned int MAX_SESSION_CHECKOUT_TRY = 10;
-
 	ControllerMainConfig mainConfig;
 	ControllerRequestConfigPtr requestConfig;
 	StringKeyTable< boost::shared_ptr<Options> > poolOptionsCache;
@@ -123,6 +111,7 @@ private:
 	HashedStaticString PASSENGER_SHOW_VERSION_IN_HEADER;
 	HashedStaticString PASSENGER_STICKY_SESSIONS;
 	HashedStaticString PASSENGER_STICKY_SESSIONS_COOKIE_NAME;
+	HashedStaticString PASSENGER_STICKY_SESSIONS_COOKIE_ATTRIBUTES;
 	HashedStaticString PASSENGER_REQUEST_OOB_WORK;
 	HashedStaticString REMOTE_ADDR;
 	HashedStaticString REMOTE_PORT;
@@ -203,9 +192,14 @@ private:
 		const AbstractSessionPtr &session, const ExceptionPtr &e);
 	void maybeSend100Continue(Client *client, Request *req);
 	void initiateSession(Client *client, Request *req);
+	void finishInitiatingSession(Client *client, Request *req);
+	static void onSessionSocketConnected(EV_P_ ev_io *io, int revents);
+	static void onSessionSocketConnectTimeout(EV_P_ ev_timer *io, int flag);
 	static void checkoutSessionLater(Request *req);
 	void reportSessionCheckoutError(Client *client, Request *req,
 		const ExceptionPtr &e);
+	void handleSessionInitiationError(Client *client, Request *req, const std::exception &e);
+	int lookupCodeFromHeader(Request *req, const char* header, int statusCode);
 	void writeRequestQueueFullExceptionErrorResponse(Client *client,
 		Request *req, const boost::shared_ptr<RequestQueueFullException> &e);
 	void writeSpawnExceptionErrorResponse(Client *client, Request *req,
@@ -213,7 +207,8 @@ private:
 	void writeOtherExceptionErrorResponse(Client *client, Request *req,
 		const ExceptionPtr &e);
 	void endRequestWithErrorResponse(Client **c, Request **r,
-		const SpawningKit::SpawnException &e);
+		const SpawningKit::SpawnException &e, int statusCode);
+	const LString* customErrorPageEnabled(Request *req);
 	bool friendlyErrorPagesEnabled(Request *req);
 
 
@@ -312,9 +307,12 @@ private:
 		Request **req);
 	void endRequestWithAppSocketReadError(Client **client, Request **req,
 		int e);
+	ServerKit::HeaderTable getHeadersWithContentType(Request *req);
+	const string getFormattedMessage(Request *req, const StaticString &body);
 	void endRequestWithSimpleResponse(Client **c, Request **r,
 		const StaticString &body, int code = 200);
 	void endRequestAsBadGateway(Client **client, Request **req);
+	void endRequestAsGatewayTimeout(Client **client, Request **req);
 	void writeBenchmarkResponse(Client **client, Request **req,
 		bool end = true);
 	bool getBoolOption(Request *req, const HashedStaticString &name,
@@ -359,9 +357,15 @@ protected:
 
 	virtual void asyncGetFromApplicationPool(Request *req,
 		ApplicationPool2::GetCallback callback);
+	virtual int getSessionSocketConnectIoWatchConditions() const;
+	virtual double getSessionSocketEffectiveConnectTimeout(Request *req) const;
 
 
 public:
+	// If you change this value, make sure that Request::sessionCheckoutTry
+	// has enough bits.
+	static constexpr unsigned int MAX_SESSION_CHECKOUT_TRY = 10;
+
 	typedef ControllerConfigChangeRequest ConfigChangeRequest;
 
 	// Dependencies
