@@ -1,9 +1,9 @@
 /*
  *  Phusion Passenger - https://www.phusionpassenger.com/
- *  Copyright (c) 2011-2017 Phusion Holding B.V.
+ *  Copyright (c) 2011-2025 Asynchronous B.V.
  *
  *  "Passenger", "Phusion Passenger" and "Union Station" are registered
- *  trademarks of Phusion Holding B.V.
+ *  trademarks of Asynchronous B.V.
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
@@ -27,17 +27,12 @@
 #define _PASSENGER_REQUEST_HANDLER_REQUEST_H_
 
 #include <ev++.h>
-#include <string>
-#include <cstring>
 
 #include <ServerKit/HttpRequest.h>
 #include <ServerKit/FdSinkChannel.h>
 #include <ServerKit/FdSourceChannel.h>
-#include <Logging.h>
+#include <LoggingKit/LoggingKit.h>
 #include <Core/ApplicationPool/Pool.h>
-#include <Core/UnionStation/Context.h>
-#include <Core/UnionStation/Transaction.h>
-#include <Core/UnionStation/StopwatchLog.h>
 #include <Core/Controller/Config.h>
 #include <Core/Controller/AppResponse.h>
 
@@ -73,7 +68,6 @@ public:
 	bool dechunkResponse: 1;
 	bool requestBodyBuffering: 1;
 	bool https: 1;
-	bool showVersionInHeader: 1;
 	bool stickySession: 1;
 
 	// Range: 0..MAX_SESSION_CHECKOUT_TRY
@@ -82,11 +76,14 @@ public:
 	bool appResponseInitialized: 1;
 	bool strip100ContinueHeader: 1;
 	bool hasPragmaHeader: 1;
+	unsigned int connectTimeout;
 
 	Options options;
 	AbstractSessionPtr session;
 	const LString *host;
-	ControllerRequestConfigCachePtr configCache;
+	ControllerRequestConfigPtr config;
+	ev_io connectedWatcher;
+	ev_timer connectedWatcherTimout;
 
 	ServerKit::FdSinkChannel appSink;
 	ServerKit::FdSourceChannel appSource;
@@ -95,13 +92,6 @@ public:
 	ServerKit::FileBufferedChannel bodyBuffer;
 	boost::uint64_t bodyBytesBuffered; // After dechunking
 
-	struct {
-		UnionStation::StopwatchLog *requestProcessing;
-		UnionStation::StopwatchLog *bufferingRequestBody;
-		UnionStation::StopwatchLog *getFromPool;
-		UnionStation::StopwatchLog *requestProxying;
-	} stopwatchLogs;
-
 	HashedStaticString cacheKey;
 	LString *cacheControl;
 	LString *varyCookie;
@@ -123,9 +113,7 @@ public:
 
 	Request()
 		: BaseHttpRequest()
-	{
-		memset(&stopwatchLogs, 0, sizeof(stopwatchLogs));
-	}
+		{ }
 
 	const char *getStateString() const {
 		switch (state) {
@@ -146,28 +134,6 @@ public:
 		}
 	}
 
-	bool useUnionStation() const {
-		return options.transaction != NULL;
-	}
-
-	void beginStopwatchLog(UnionStation::StopwatchLog **stopwatchLog, const char *id, const char *nameAndData = NULL) {
-		if (options.transaction != NULL) {
-			*stopwatchLog = new UnionStation::StopwatchLog(options.transaction, id, nameAndData);
-		}
-	}
-
-	void endStopwatchLog(UnionStation::StopwatchLog **stopwatchLog, bool success = true) {
-		if (success && *stopwatchLog != NULL) {
-			(*stopwatchLog)->success();
-		}
-		delete *stopwatchLog;
-		*stopwatchLog = NULL;
-	}
-
-	void logMessage(const StaticString &message) {
-		options.transaction->message(message);
-	}
-
 	DEFINE_SERVER_KIT_BASE_HTTP_REQUEST_FOOTER(Passenger::Core::Request);
 };
 
